package edu.ehu.gb.tap;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author Clase
 */
public class Babel {
    private static final String INDEX_FILENAME="languages.txt";
    private static final String SUBDIR="txt/";
    private static final String EXTENSION=".txt";
    
    private final Map<String,Language> lenguajes=new HashMap<>();  
    
    
    Babel(String dirname) throws IOException {
        try (BufferedReader br=new BufferedReader(new FileReader(dirname+INDEX_FILENAME));){
            String linea;
            while ((linea=br.readLine())!=null){
                if (linea.trim().length()==0) continue;
                String id=linea.substring(0,linea.indexOf(' '));
                String descriptor=linea.substring(linea.indexOf(' ')+1);
                try{
                    lenguajes.put(id,new Language(id,descriptor,dirname+SUBDIR+id+EXTENSION));
                } catch(IOException ex) {
                        System.err.println("No he podido adquirir el lenguaje: "+linea);
                }
            }
        }
    }
  
    public Language[] findMostSimilars(String id,int n){
        Language l1=lenguajes.get(id);
      
        class LD implements Comparable{
                String id; double similitud;

                public LD(String id, double similitud) {
                    this.id = id;
                    this.similitud = similitud;
                }

        @Override
        public int compareTo(Object o) {
        return -Double.compare(similitud, ((LD)o).similitud); }

            @Override
            public String toString() {
                return "LD{" + "id=" + id + ", similitud=" + similitud + '}';
            }
        }

        LD[] similitudes=new LD[lenguajes.size()];
        int i=0;
        for(String l2Key:lenguajes.keySet())
             similitudes[i++]=new LD(l2Key,Language.similarity(l1, lenguajes.get(l2Key)));
            
        Arrays.sort(similitudes);
        
        Language[] out=new Language[n<similitudes.length?n:similitudes.length-1];
        for (int j=0;j<out.length;j++) out[j]=lenguajes.get(similitudes[j+1].id); 
        
        return out; 
    }
    
}

